@@ -2,7 +2,7 @@
  *
  *  Name:       common.c
  *
- *  Version:    2.6-2
+ *  Version:    2.9-1
  *
  *  Purpose:    Common definitions for RNetCDF functions
  *
@@ -44,7 +44,7 @@ int
 R_nc_check(int status)
 {
   if (status != NC_NOERR) {
-    error (nc_strerror (status));
+    error ("%s", nc_strerror (status));
   }
   return status;
 }
@@ -59,6 +59,15 @@ R_nc_strcmp (SEXP var, const char *str)
 }
 
 
+size_t
+R_nc_strnlen (const char *str, char chr, size_t maxlen)
+{
+  char *nullchr;
+  nullchr = memchr(str, (int) chr, maxlen);
+  return (nullchr == NULL) ? maxlen : ((size_t) (nullchr - str));
+}
+
+
 int
 R_nc_inherits (SEXP var, const char *class)
 {
