@@ -2,7 +2,7 @@
  *
  *  Name:       type.c
  *
- *  Version:    2.6-2
+ *  Version:    2.9-1
  *
  *  Purpose:    NetCDF type functions for RNetCDF
  *
@@ -366,9 +366,10 @@ R_nc_inq_type (SEXP nc, SEXP type, SEXP fields)
         resultnames = PROTECT(allocVector (STRSXP, 7));
         setAttrib (result, R_NamesSymbol, resultnames);
         UNPROTECT(1);
-        SET_STRING_ELT (resultnames, 4, mkChar ("offset"));
-        SET_STRING_ELT (resultnames, 5, mkChar ("subtype"));
-        SET_STRING_ELT (resultnames, 6, mkChar ("dimsizes"));
+        SET_STRING_ELT (resultnames, 4, PROTECT(mkChar ("offset")));
+        SET_STRING_ELT (resultnames, 5, PROTECT(mkChar ("subtype")));
+        SET_STRING_ELT (resultnames, 6, PROTECT(mkChar ("dimsizes")));
+        UNPROTECT(3);
 
 	/* Read named vectors of offsets, typenames, list of array dimensions */
 	offsets = PROTECT(allocVector (REALSXP, nfields));
@@ -389,10 +390,11 @@ R_nc_inq_type (SEXP nc, SEXP type, SEXP fields)
 	for (ii=0; ii < imax; ii++) {
 	  R_nc_check (nc_inq_compound_field (ncid, xtype, ii, fieldname,
 		      &offset, &subtype, &ndims, NULL));
-	  SET_STRING_ELT (fieldnames, ii, mkChar (fieldname));
+	  SET_STRING_ELT (fieldnames, ii, PROTECT(mkChar (fieldname)));
 	  REAL (offsets)[ii] = offset;
 	  R_nc_check (R_nc_type2str (ncid, subtype, subname));
-	  SET_STRING_ELT (subnames, ii, mkChar (subname));
+	  SET_STRING_ELT (subnames, ii, PROTECT(mkChar (subname)));
+          UNPROTECT(2);
 	  if (ndims > 0) {
 	    dimsize = PROTECT(allocVector (INTSXP, ndims));
 	    R_nc_check (nc_inq_compound_fielddim_sizes (
@@ -409,7 +411,8 @@ R_nc_inq_type (SEXP nc, SEXP type, SEXP fields)
         UNPROTECT(1);
       }
 
-      SET_VECTOR_ELT (result, 2, mkString ("compound"));
+      SET_VECTOR_ELT (result, 2, PROTECT(mkString ("compound")));
+      UNPROTECT(1);
       break;
     case NC_ENUM:
       R_nc_check (R_nc_type2str (ncid, basetype, basename));
@@ -419,8 +422,8 @@ R_nc_inq_type (SEXP nc, SEXP type, SEXP fields)
 
 	resultnames = PROTECT(allocVector (STRSXP, 6));
         setAttrib (result, R_NamesSymbol, resultnames);
-        UNPROTECT(1);
-	SET_STRING_ELT (resultnames, 5, mkChar ("value"));
+	SET_STRING_ELT (resultnames, 5, PROTECT(mkChar ("value")));
+        UNPROTECT(2);
 
 	/* Read named vector of member values */
 	fieldnames = PROTECT(allocVector (STRSXP, nfields));
@@ -432,7 +435,8 @@ R_nc_inq_type (SEXP nc, SEXP type, SEXP fields)
 	imax = nfields; // netcdf member index is int
 	for (ii=0; ii < imax; ii++, cval+=size) {
 	  R_nc_check (nc_inq_enum_member (ncid, xtype, ii, fieldname, cval));
-	  SET_STRING_ELT (fieldnames, ii, mkChar (fieldname));
+	  SET_STRING_ELT (fieldnames, ii, PROTECT(mkChar (fieldname)));
+          UNPROTECT(1);
 	}
 	R_nc_c2r (&io);
 	SET_VECTOR_ELT (result, 5, values);
@@ -446,31 +450,32 @@ R_nc_inq_type (SEXP nc, SEXP type, SEXP fields)
         UNPROTECT(1);
       }
 
-      SET_VECTOR_ELT (result, 2, mkString ("enum"));
-      SET_VECTOR_ELT (result, 4, mkString (basename));
-      SET_STRING_ELT (resultnames, 4, mkChar ("basetype"));
+      SET_VECTOR_ELT (result, 2, PROTECT(mkString ("enum")));
+      SET_VECTOR_ELT (result, 4, PROTECT(mkString (basename)));
+      SET_STRING_ELT (resultnames, 4, PROTECT(mkChar ("basetype")));
+      UNPROTECT(3);
       break;
     case NC_OPAQUE:
 
       result = PROTECT(allocVector (VECSXP, 4));
-      SET_VECTOR_ELT (result, 2, mkString ("opaque"));
+      SET_VECTOR_ELT (result, 2, PROTECT(mkString ("opaque")));
 
       resultnames = PROTECT(allocVector (STRSXP, 4));
       setAttrib (result, R_NamesSymbol, resultnames);
-      UNPROTECT(1);
+      UNPROTECT(2);
 
       break;
     case NC_VLEN:
       R_nc_check (R_nc_type2str (ncid, basetype, basename));
 
       result = PROTECT(allocVector (VECSXP, 5));
-      SET_VECTOR_ELT (result, 2, mkString ("vlen"));
-      SET_VECTOR_ELT (result, 4, mkString (basename));
+      SET_VECTOR_ELT (result, 2, PROTECT(mkString ("vlen")));
+      SET_VECTOR_ELT (result, 4, PROTECT(mkString (basename)));
 
       resultnames = PROTECT(allocVector (STRSXP, 5));
       setAttrib (result, R_NamesSymbol, resultnames);
-      UNPROTECT(1);
-      SET_STRING_ELT (resultnames, 4, mkChar ("basetype"));
+      SET_STRING_ELT (resultnames, 4, PROTECT(mkChar ("basetype")));
+      UNPROTECT(4);
 
       break;
     default:
@@ -481,23 +486,23 @@ R_nc_inq_type (SEXP nc, SEXP type, SEXP fields)
   /*-- Built-in types ---------------------------------------------------------*/
 
       result = PROTECT(allocVector (VECSXP, 4));
-      SET_VECTOR_ELT (result, 2, mkString ("builtin"));
+      SET_VECTOR_ELT (result, 2, PROTECT(mkString ("builtin")));
 
       resultnames = PROTECT(allocVector (STRSXP, 4));
       setAttrib (result, R_NamesSymbol, resultnames);
-      UNPROTECT(1);
+      UNPROTECT(2);
   }
 
   /*-- Common components of output list ----------------------------------------------*/
-  SET_VECTOR_ELT (result, 0, ScalarInteger (xtype));
-  SET_VECTOR_ELT (result, 1, mkString (typename));
-  SET_VECTOR_ELT (result, 3, ScalarReal (size));
+  SET_VECTOR_ELT (result, 0, PROTECT(ScalarInteger (xtype)));
+  SET_VECTOR_ELT (result, 1, PROTECT(mkString (typename)));
+  SET_VECTOR_ELT (result, 3, PROTECT(ScalarReal (size)));
 
-  SET_STRING_ELT (resultnames, 0, mkChar ("id"));
-  SET_STRING_ELT (resultnames, 1, mkChar ("name"));
-  SET_STRING_ELT (resultnames, 2, mkChar ("class"));
-  SET_STRING_ELT (resultnames, 3, mkChar ("size"));
+  SET_STRING_ELT (resultnames, 0, PROTECT(mkChar ("id")));
+  SET_STRING_ELT (resultnames, 1, PROTECT(mkChar ("name")));
+  SET_STRING_ELT (resultnames, 2, PROTECT(mkChar ("class")));
+  SET_STRING_ELT (resultnames, 3, PROTECT(mkChar ("size")));
 
-  UNPROTECT(1);
+  UNPROTECT(8);
   return result;
 }
